/*-----------------------------------------------------------------------

File  : clb_memory.c

Author: Stephan Schulz

  Memory management. The implemented routines are very simple, because
  they make use of the standard memory management for reorganization.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Thu Aug 14 10:00:35 MET DST 1997
    new

-----------------------------------------------------------------------*/

#include "clb_memory.h"


/*-----------------------------------------------------------------------*/
/*                         Globale Variable                              */
/*-----------------------------------------------------------------------*/

/* This global variable is set whenever malloc() failed and triggered
   a free memory reorganization. User programs may examine this
   variable to take certain measures (and may reset it if they think
   that they freed significant amounts of memory). However, this is
   somewhat discouraged - do you really want your program to depend in
   pretty complex ways on uncontrolable features like the amount of
   free memory available to your process? _I_ only use it for some
   measurements ;-)  StS */

bool MemIsLow = false;

#define MEM_ARR_SIZE 1024
#define MEM_FREE_PATTERN 0xFAFBFAFA
#define MEM_RSET_PATTERN 0x00000000

static Mem_p free_mem_list[MEM_ARR_SIZE] = {NULL};

#ifdef CLB_MEMORY_DEBUG 
static long size_malloc_mem = 0;
static long size_malloc_count = 0;
static long size_free_mem = 0;
static long size_free_count = 0;
       long clb_free_count = 0;
static long secure_malloc_count = 0;
static long secure_malloc_mem = 0;
static long secure_realloc_count = 0;
static long secure_realloc_m_count = 0;
static long secure_realloc_f_count = 0;
#endif

/*-----------------------------------------------------------------------*/
/*                  Exportierte Funktionen                               */
/*-----------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: SizeMallocReal()
//
//   Returns a block of memory sized size using the internal
//   free-list. This block is freeable with free(), and in all
//   respects behaves like a normal malloc'ed block. 
//
// Global Variables: free_mem_list[] 
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

void* SizeMallocReal(int size)
{
   Mem_p   handle;
   int    index;
   
   index = size - sizeof(MemCell);
   
   if((index>=0) && (index<MEM_ARR_SIZE) && free_mem_list[index])
   {
      assert(free_mem_list[index]->test == MEM_FREE_PATTERN);
      assert((free_mem_list[index]->test = MEM_RSET_PATTERN, true));
      handle = (void*)(free_mem_list[index]);
      free_mem_list[index] = free_mem_list[index]->next;
   } 
   else
   {
      handle = SecureMalloc(size);
#ifndef NDBUG
      if((index>=0) && (index<MEM_ARR_SIZE))
      {
	 assert((handle->test = MEM_RSET_PATTERN, true));
      }
#endif
   }
   #ifdef CLB_MEMORY_DEBUG 
   size_malloc_mem+=size;
   size_malloc_count++;
   #endif
   #ifdef CLB_MEMORY_DEBUG2
   printf("\nBlock %p A: size %d\n", handle, size);
   #endif
   return handle;
}   


/*-----------------------------------------------------------------------
//
// Function: SizeFreeReal()
//
//  Returns a block sized size. Note: size has to be exact - you
//  should only give blocks to SizeFree() that have been allocated
//  with malloc(size) or SizeMalloc(size). Giving blocks that are to
//  big wastes memory, blocks that are to small will result in more
//  serious trouble (segmentation fault)
//
// Global Variables: free_mem_list[]
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

void SizeFreeReal(void* junk, int size)
{
   int    index;
   
   assert(junk!=NULL);

   #ifdef CLB_MEMORY_DEBUG2
   printf("\nBlock %p D: size %d\n", junk, size);
   #endif

   index = size - sizeof(MemCell);

   if(index>=0 && index<MEM_ARR_SIZE)
   {
      ((Mem_p)junk)->next = free_mem_list[index];
      free_mem_list[index] = (Mem_p)junk;
      assert(free_mem_list[index]->test != MEM_FREE_PATTERN);
      assert(free_mem_list[index]->test = MEM_FREE_PATTERN);
   } 
   else
   {
      FREE(junk);
   }
   #ifdef CLB_MEMORY_DEBUG 
   size_free_mem+=size;
   size_free_count++;
   #endif
}   

/*-----------------------------------------------------------------------
//
// Function: MemFlushFreeList()
//
//   Returns all memory kept in free_mem_list[] to the operation
//   system. This is useful if a very different memory access pattern
//   is expected (SizeFree() never reorganizes the memory
//   automatically). 
//
// Global Variables: free_mem_list[] 
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

void MemFlushFreeList()
{
   int f;
   void* handle;

   VERBOUT("MemFlushFreeList() called for cleanup or reorganization\n");
   for(f = 0;f<MEM_ARR_SIZE;f++) 
   {
      while(free_mem_list[f])
      {
	 handle = (void*)free_mem_list[f];
	 free_mem_list[f] = free_mem_list[f]->next;
	 FREE(handle);
      }
   }
}

/*-----------------------------------------------------------------------
//
// Function: SecureMalloc()
//
//  Returns a pointer to an unused memory block sized size. If
//  available, a block from free_mem_list is returned, otherwise a
//  fresh one is allocated. If necessary, the reorganization of
//  free_mem_list is triggered, if no memory is available, an error
//  will be produced.
//
// Global Variables: free_mem_list
//
// Side Effects    : Memory operations, possibly error
//
/----------------------------------------------------------------------*/

void* SecureMalloc(int size)
{
   void* handle;

   #ifdef CLB_MEMORY_DEBUG 
   secure_malloc_count++;
   secure_malloc_mem += size;
   #endif

   handle = (void*)malloc(size);

   if(!handle)
   {    /* malloc has no memory left  */
      MemIsLow = true;
      MemFlushFreeList(); /* Return own freelist */
      
      handle = (void*)malloc(size);
      
      if(!handle)
      {   /*  Still nothing...*/
#ifdef PRINT_NOMEM_STDOUT
	 PrintRusage(stdout);
	 fprintf(stdout, "# Resource limit exceeded (memory)\n");
#endif
         Error("Out of Memory", OUT_OF_MEMORY);
      }
   }
   return handle;
}

/*-------------------------------------------------------------------------
//
// Function: SecureRealloc()
//
//   Imitates Realloc, but reorganizes free_mem_list to get new memory
//   if the block has to be moved and no memory is available. Will
//   terminate with OUT_OF_MEMORY if no memory is found. 
//
// Global Variables: -
//
// Side Effect     : Via SecureMalloc()
//
//-----------------------------------------------------------------------*/

void* SecureRealloc(void *ptr, int size)
{
   void* handle;
   
#ifdef CLB_MEMORY_DEBUG 
   secure_realloc_count++;
   if(ptr && !size)
   {
      secure_realloc_f_count++;
   }
   else if(!ptr && size)
   {
      secure_realloc_m_count++;
   }
#endif
   /* SunOS realloc() is broken, so here is a stupid workaround...*/

   handle = ptr?realloc(ptr,size):malloc(size);
   if(!handle && size!=0)
   {
      MemFlushFreeList();
      handle = ptr?realloc(ptr,size):malloc(size);
      if(!handle)
      {   /*  Still nothing...*/
	 Error("Out of Memory", OUT_OF_MEMORY);
      }
   }
   return handle;
}

/*-----------------------------------------------------------------------
//
// Function: SecureStrdup()
//
//   Implements the functionality of strdup, but uses SecureMalloc()
/    for the memory handling. 
//
// Global Variables: -
//
// Side Effects    : By SecureMalloc()
//
/----------------------------------------------------------------------*/

char* SecureStrdup(char* source)
{
   char* handle;

   handle = (char*)SecureMalloc(strlen(source)+1);
   strcpy(handle,source);

   return handle;
}

/*-----------------------------------------------------------------------
//
// Function: AllocIntArray()
//
//   Return a pointer to  a freshly allocated, 0-initialized block of
//   longs. 
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

long* IntArrayAlloc(int size)
{
   long *res;
   int i;

   res = SizeMalloc(size*sizeof(long));

   for(i=0; i<size; i++)
   {
      res[i]=0;
   }
   return res;
}




#ifdef CLB_MEMORY_DEBUG

/*-----------------------------------------------------------------------
//
// Function: MemDebugPrintStats()
//
//   Print information about allocated and deallocated memory.
//
// Global Variables: size_malloc_mem, size_malloc_count,
//                   size_free_mem, size_free_count
//
// Side Effects    : Output
//
/----------------------------------------------------------------------*/

void MemDebugPrintStats(FILE* out)
{
   fprintf(out, 
	   "\n# -------------------------------------------------\n");
   fprintf(out,
	   "# Total SizeMalloc()ed memory: %ld Bytes (%ld requests)\n",
	   size_malloc_mem, size_malloc_count);
   fprintf(out,
	   "# Total SizeFree()ed   memory: %ld Bytes (%ld requests)\n",
	   size_free_mem, size_free_count);
   fprintf(out, 
	   "# New requests: %6ld (%6ld by SecureMalloc(), %6ld by SecureRealloc())\n",
	   secure_malloc_count+secure_realloc_m_count,
	   secure_malloc_count, secure_realloc_m_count);
   fprintf(out, 
	   "# Total SecureMalloc()ed memory: %ld Bytes\n", secure_malloc_mem);
   fprintf(out,  
	   "# Returned:    %6ld (%6ld by FREE(),         %6ld by SecureRealloc())\n",
	   clb_free_count+secure_realloc_f_count,
	   clb_free_count , secure_realloc_f_count);
   fprintf(out,
	   "# SecureRealloc(ptr): %6ld (%6ld Allocs, %6ld Frees, %6ld Reallocs)\n",
	   secure_realloc_count, secure_realloc_m_count,
	   secure_realloc_f_count,
	   secure_realloc_count-(secure_realloc_m_count+secure_realloc_f_count)); 
   fprintf(out,
	   "# -------------------------------------------------\n\n");
}

#endif

/*-----------------------------------------------------------------------*/
/*                       Ende des Files                                  */
/*-----------------------------------------------------------------------*/


