/*-----------------------------------------------------------------------

File  : cco_paramodulation.c

Author: Stephan Schulz

Contents
 
  Controling paramodulation inferences.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Mon Jun  8 18:26:14 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "cco_paramodulation.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//
// Function: ComputeClauseClauseParamodulants()
//
//   Compute all paramodulants between clause and with, with terms
//   from bank, and put them into store. If units_only is true,
//   overlap only with unit-clauses (but into arbitrary ones). Returns
//   number of paramodulants. 
//
// Global Variables: -
//
// Side Effects    : As above...
//
/----------------------------------------------------------------------*/

long ComputeClauseClauseParamodulants(TB_p bank, OCB_p ocb, Clause_p
				      clause, Clause_p parent_alias,
				      Clause_p with, ClauseSet_p
				      store, VarBank_p freshvars,
				      ParamodStrategy strategy, long*
				      idents)
{
   Clause_p    paramod;
   Term_p      test;
   ClausePos_p pos1, pos2;
   long        paramod_count = 0;

   pos1 = ClausePosAlloc();
   pos2 = ClausePosAlloc();

   DEBUGMARK(256, "ComputeClauseClauseParamodulants()...\n");
   DEBUG(256,
	 ClausePrint(stdout, clause, true);
	 printf(" =|=|= ");
	 ClausePrint(stdout, parent_alias, true); 	 
	 printf("\n");
	 ClausePrint(stdout, with, true);
	 printf("\n");
	 );

   test = ClausePosFirstParamodPair(clause, pos1, with, pos2,
				    normal, strategy);
   
   while(test)
   {
      assert(TermPosIsTopPos(pos1->pos));
      paramod = ClauseOrderedParamod(bank, ocb, pos1, pos2,
				     freshvars);
      if(paramod)	    
      {
	 paramod_count++;
	 paramod->ident = ++(*idents);
	 paramod->parent1 = parent_alias;	    
	 paramod->proof_size  =
	    parent_alias->proof_size+with->proof_size+1;
	 paramod->proof_depth = MAX(parent_alias->proof_depth,
				    with->proof_depth)+1;
	 ClauseSetTPTPType(paramod,
			   ClauseQueryTPTPType(parent_alias));
	 
	 ClauseRegisterChild(parent_alias, paramod);
	 if(parent_alias!=with)
	 {
	    paramod->parent2 = with;
	    
	    ClauseSetTPTPType(paramod,
			      TPTPTypesCombine(
				 ClauseQueryTPTPType(paramod),
				 ClauseQueryTPTPType(with)));
	    
	    ClauseRegisterChild(with, paramod);
	 }
	 ClauseDefaultDocumentCreation(paramod, CC_PARAMOD);
	 ClauseSetInsert(store, paramod);
      }
      test = ClausePosNextParamodPair(pos1, pos2, normal, strategy);
   }
   /* Paramod clause into with - no top positions this time ;-) */

   test = ClausePosFirstParamodPair(with, pos1, clause, pos2, true,
				    strategy);
      
   while(test)
   {
      assert(TermPosIsTopPos(pos1->pos));
      paramod = ClauseOrderedParamod(bank, ocb, pos1, pos2, freshvars);
      if(paramod)
      {
	 paramod_count++;
	 paramod->ident = ++(*idents);
	 paramod->parent1 = parent_alias;
	 paramod->proof_size  =
	    parent_alias->proof_size+with->proof_size+1;
	 paramod->proof_depth = MAX(parent_alias->proof_depth,
				    with->proof_depth)+1;
	 ClauseSetTPTPType(paramod,
			   ClauseQueryTPTPType(parent_alias));
	 ClauseRegisterChild(parent_alias, paramod);
	 if(parent_alias!=with)
	 {
	    paramod->parent2 = with;
	    ClauseSetTPTPType(paramod,
			      TPTPTypesCombine(
				 ClauseQueryTPTPType(paramod),
				 ClauseQueryTPTPType(with)));
	    ClauseRegisterChild(with, paramod);
	 }
	 ClauseDefaultDocumentCreation(paramod, CC_PARAMOD);
	 ClauseSetInsert(store, paramod);
      }
      test = ClausePosNextParamodPair(pos1, pos2, true, strategy);
   }
   ClausePosFree(pos1);
   ClausePosFree(pos2);
   
   return paramod_count;
}


/*-----------------------------------------------------------------------
//
// Function: ComputeAllParamodulants()
//
//   Compute all paramodulants between clause and with_set, put them
//   into store.
//
// Global Variables: -
//
// Side Effects    : By called functions, memory manangement.
//
/----------------------------------------------------------------------*/

long ComputeAllParamodulants(TB_p bank, OCB_p ocb, Clause_p clause,
			     Clause_p parent_alias, ClauseSet_p
			     with_set, ClauseSet_p store, VarBank_p
			     freshvars, ParamodStrategy strategy,
			     long* idents)
{
   Clause_p handle;
   long     paramod_count = 0;

   for(handle = with_set->anchor->succ; handle != with_set->anchor;
       handle = handle->succ)
   {
      paramod_count +=
	 ComputeClauseClauseParamodulants(bank, ocb, clause,
					  parent_alias, handle,
					  store, freshvars,
					  strategy, idents);
   }
   return paramod_count;
}



/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


