/*-----------------------------------------------------------------------

File  : generate_examples.c

Author: Stephan Schulz

Contents
 
  Read the output of proofanalyze and the -l3 or -l4 output of E and
  print the selected clauses with their statistics.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Wed Mar 31 18:05:45 MET DST 1999
    New

-----------------------------------------------------------------------*/

#include <cio_commandline.h>
#include <cio_output.h>
#include <ccl_clausesets.h>
#include <can_treeanalyze.h>


/*---------------------------------------------------------------------*/
/*                  Data types                                         */
/*---------------------------------------------------------------------*/

#define VERSION "0.1dev"

typedef enum
{
   OPT_NOOPT=0,
   OPT_HELP,
   OPT_VERSION,
   OPT_VERBOSE,
   OPT_OUTPUT,
   OPT_PRINT_EVAL,
   OPT_NOPRINT_STATS
}OptionCodes;



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/

OptCell opts[] =
{
   {OPT_HELP, 
    'h', "help", 
    NoArg, NULL,
    "Print a short description of program usage and options."},

   {OPT_VERSION,
    '\0', "version",
    NoArg, NULL,
    "Print the version number of the program."},

   {OPT_VERBOSE, 
    'v', "verbose", 
    OptArg, "1",
    "Verbose comments on the progress of the program. This differs "
    "from the output level (below) in that technical information is "
    "printed to stderr, while the output level determines which "
    "logical manipulations of the clauses are printed to stdout."},

   {OPT_OUTPUT,
    'o', "output-file",
    ReqArg, NULL,
   "Redirect output into the named file."},

   {OPT_PRINT_EVAL,
    'e', "print-clause-at-evaluation",
    NoArg, NULL,
    "Print a the selected clauses in the form they are evaluated"
    " (default is to print the selected clauses whenever they are"
    "modified by an inference)."},

   {OPT_NOPRINT_STATS,
    'i', "inhibit-status-information",
    NoArg, NULL,
    "Do not print statistical information for clauses."},
  
   {OPT_NOOPT,
    '\0', NULL,
    NoArg, NULL,
    NULL}
};

char   *outname = NULL;
bool   print_eval = false,
       print_stats = true;


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/

CLState_p process_options(int argc, char* argv[]);
void print_help(FILE* out);

/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/


int main(int argc, char* argv[])
{
   Scanner_p       in;    
   int             i;
   CLState_p       state;
   InfState_p      infstate;   

   assert(argv[0]);

   InitError("generate_examples");

   /* printf("TermCell: %d\nClauseCell: %d\nCompClauseCell:"
	  "%d\nPTreeCell: %d\n",
	  sizeof(TermCell), sizeof(ClauseCell),
	  sizeof(CompClauseCell), sizeof(PTreeCell)); */

   state = process_options(argc, argv);

   GlobalOut = OutOpen(outname);
   OutputLevel = 0;
   
   if(state->argc ==  0)
   {
      CLStateInsertArg(state, "-");
   }

   ClausesHaveLocalVariables = false; /* We need consistent
					 name->variable mappings in
					 this application! */
   infstate = InfStateAlloc();
   infstate->print_clause_evals = print_eval;
   infstate->print_clause_mods  = !print_eval;
   infstate->print_clause_stats = print_stats;
   for(i=0; state->argv[i]; i++)
   {
      in = CreateScanner(StreamTypeFile, state->argv[i] , true, NULL);
      
      ClauseSetInfoParse(in, &(infstate->watch_clauses));
      InfStateInfListParse(in, infstate);
      
      DestroyScanner(in);
   }
   InfStateFree(infstate);

   CLStateFree(state);
   
   fflush(GlobalOut);
   OutClose(GlobalOut);
   
#ifdef CLB_MEMORY_DEBUG
   MemFlushFreeList();
   MemDebugPrintStats(stdout);
#endif
   
   return 0;
}


/*-----------------------------------------------------------------------
//
// Function: process_options()
//
//   Read and process the command line option, return (the pointer to)
//   a CLState object containing the remaining arguments.
//
// Global Variables: 
//
// Side Effects    : Sets variables, may terminate with program
//                   description if option -h or --help was present
//
/----------------------------------------------------------------------*/

CLState_p process_options(int argc, char* argv[])
{
   Opt_p handle;
   CLState_p state;
   char*  arg;
   
   state = CLStateAlloc(argc,argv);
   
   while((handle = CLStateGetOpt(state, &arg, opts)))
   {
      switch(handle->option_code)
      {
      case OPT_VERBOSE:
	    Verbose = CLStateGetIntArg(handle, arg);
	    break;
      case OPT_HELP: 
	    print_help(stdout);
	    exit(NO_ERROR);
	    break;
      case OPT_VERSION:
	    printf("dep_analyze " VERSION "\n");
	    exit(NO_ERROR);
	    break;
      case OPT_OUTPUT:
	    outname = arg;
	    break;
      case OPT_PRINT_EVAL:
	    print_eval = true;
	    break;	    
      case OPT_NOPRINT_STATS:
	    print_stats = false;
	    break;
     default:
	 assert(false);
	 break;
      }
   }
   return state;
}

void print_help(FILE* out)
{
   fprintf(out, "\n\
\n\
generate_examples " VERSION "\n\
\n\
Usage: generate_examples [options] [files]\n\
\n\
Parse a clause selection and a full E inference listing (possibly\n\
spread over multiple files), and generate training examples\n\
corresponding to the selected clauses.\n"); 
   PrintOptions(stdout, opts);
   fprintf(out, "\n\
Copyright 1998 by Stephan Schulz, schulz@informatik.tu-muenchen.de\n\
\n\
This program is a part of the support structure for the E equational\n\
theorem prover. You can find the latest version of the E distribution\n\
as well as additional information at\n\
http://wwwjessen.informatik.tu-muenchen.de/~schulz/WORK/eprover.html.\n\
\n\
This program is free software; you can redistribute it and/or modify\n\
it under the terms of the GNU General Public License as published by\n\
the Free Software Foundation; either version 2 of the License, or\n\
(at your option) any later version.\n\
\n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n\
\n\
You should have received a copy of the GNU General Public License\n\
along with this program (it should be contained in the top level\n\
directory of the distribution in the file COPYING); if not, write to\n\
the Free Software Foundation, Inc., 59 Temple Place, Suite 330,\n\
Boston, MA  02111-1307 USA\n\
\n\
The original copyright holder can be contacted as\n\
\n\
Stephan Schulz\n\
Technische Universitaet Muenchen\n\
Fakultaet fuer Informatik\n\
Arcisstrasse 20\n\
D-80290 Muenchen\n\
Germany\n\
");

}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


